@@ -9,6 +9,7 @@ import sys
 import textwrap
 from argparse import ArgumentParser, RawDescriptionHelpFormatter, RawTextHelpFormatter
 from dataclasses import MISSING, dataclass, field, is_dataclass
+from enum import Enum
 from itertools import zip_longest
 from pathlib import Path
 from subprocess import PIPE, run
@@ -100,7 +101,7 @@ def _locate_config(path: Optional[str]):
     if path and os.path.isabs(path):
         return Path(path)
     if path and re.fullmatch("[A-Za-z0-9]+", path) is None:
-        print_err("--config contains forbiden chars. Please provide an absolute path")
+        print_err("--config contains forbidden chars. Please provide an absolute path")
         sys.exit(2)
 
     filename = f"monitor-commander-{path}.yml" if path else "monitor-commander.yml"
@@ -167,7 +168,7 @@ def _detect_monitors(config: _Config) -> List[Dict[str, _MonitorProperties]]:
         if i != 0:
             print_err("retry...")
             sleep(1)
-        proc = run([*_ddcutil(config), "detect", "--async"], stdout=PIPE, text=True, check=False)
+        proc = run([*_ddcutil(config), "detect", "--async", "--nousb"], stdout=PIPE, text=True, check=False)
         out = proc.stdout
         if proc.returncode != 0:
             print(out)
@@ -388,12 +389,45 @@ def _action_monitors(config: _Config):
 
 
 def _elevate():
+    print_err("Action require root privileges")
+    command = ["sudo", sys.executable, *sys.argv]
+    os.execvp(command[0], command)
+
+
+def _ddcutil_need_root():
+    for device in Path("/dev").glob("i2c-*"):
+        if not os.access(device, os.W_OK, effective_ids=os.access in os.supports_effective_ids):
+            return True
+    return False
+
+
+def _config_need_root(read_only: bool):
+    access = os.R_OK if read_only else os.W_OK
+    config = _RUNTIME_CONFIG_PATH
+    while not config.exists():
+        config = config.parent
+    return not os.access(config, access, effective_ids=os.access in os.supports_effective_ids)
+
+
+class ElevateType(Enum):
+    DDCUTIL = 0
+    READ_CONF = 1
+    WRITE_CONF = 2
+
+
+def _elevate_if_needed(elevate_type: ElevateType):
     if os.getuid() == 0:
         return
 
-    print_err("Action require root privileges")
-    command = ["sudo", sys.executable, *sys.argv]
-    os.execlp(command[0], *command)
+    if elevate_type == ElevateType.DDCUTIL:
+        needroot = _ddcutil_need_root()
+    elif elevate_type == ElevateType.READ_CONF:
+        needroot = _config_need_root(True)
+    elif elevate_type == ElevateType.WRITE_CONF:
+        needroot = _config_need_root(False)
+
+    if needroot:
+        _elevate()
 
 
 def _help_parse_type(type_class: Any):
@@ -522,9 +556,20 @@ class HelpFormatter(RawTextHelpFormatter, RawDescriptionHelpFormatter):
     pass
 
 
+def _get_argv():
+    argv = sys.argv
+
+    # When completing arguments, sys.argv will contain only the script name
+    # We need to parse COMP_LINE to get the "real" arguments
+    if "_ARGCOMPLETE" in os.environ and "COMP_LINE" in os.environ:
+        _, _, _, argv, _ = split_line(os.environ["COMP_LINE"])
+
+    return argv
+
+
 def main():
     config_parser = ArgumentParser(add_help=False)
-    config_parser.add_argument(
+    argument = config_parser.add_argument(
         "--config",
         help=textwrap.dedent(
             f"""
@@ -539,12 +584,11 @@ def main():
         )
         .strip()
         .format(format="\n".join(_get_help_dataclass(_Config))),
-    ).completer = FilesCompleter  # type: ignore # Missing stub
+    )
+    setattr(argument, "completer", FilesCompleter)
 
-    argv = sys.argv
-    if "_ARGCOMPLETE" in os.environ and "COMP_LINE" in os.environ and split_line:
-        _, _, _, argv, _ = split_line(os.environ["COMP_LINE"])
-    args, _ = config_parser.parse_known_args(argv[1:])
+    # We need to pre-parse the command line as other completions depend on the config file
+    args, _ = config_parser.parse_known_args(_get_argv()[1:])
     config = _load_config(args.config)
 
     parser = ArgumentParser(parents=[config_parser], formatter_class=HelpFormatter)
@@ -558,7 +602,7 @@ def main():
     helptxt = "Probe connected monitors"
     subparser = subparsers.add_parser("monitors", help=helptxt, description=helptxt)
 
-    helptxt = "Freeze one or multiple monitors. The monitor will not be set unless explicitelly specified with the --monitor arg"
+    helptxt = "Freeze one or multiple monitors. The monitor will not be set unless explicitly specified with the --monitor arg"
     subparser = subparsers.add_parser("freeze", help=helptxt, description=helptxt)
     subparser.add_argument("--monitor", **monitor_argument)
 
@@ -584,7 +628,7 @@ def main():
         action="store_false",
         help="Do not verify values after setting them",
     )
-    subparser.add_argument(
+    argument = subparser.add_argument(
         "preset",
         help=textwrap.dedent(
             """
@@ -592,26 +636,28 @@ def main():
                 Value must be declared in the configuration file.
             """,
         ).strip(),
-    ).completer = ChoicesCompleter(  # type: ignore # Missing stub
-        {preset for monitor in config.monitors for preset in monitor.presets}
+    )
+    setattr(
+        argument, "completer", ChoicesCompleter({preset for monitor in config.monitors for preset in monitor.presets})
     )
 
     autocomplete(parser)
     args = parser.parse_args()
 
     if args.action == "monitors":
-        _elevate()
+        _elevate_if_needed(ElevateType.DDCUTIL)
         _action_monitors(config)
     elif args.action == "freeze":
-        _elevate()
+        _elevate_if_needed(ElevateType.WRITE_CONF)
         _action_freeze(config, args.monitor, True)
     elif args.action == "unfreeze":
-        _elevate()
+        _elevate_if_needed(ElevateType.WRITE_CONF)
         _action_freeze(config, args.monitor, False)
     elif args.action == "status":
+        _elevate_if_needed(ElevateType.READ_CONF)
         _action_status()
     elif args.action == "set":
-        _elevate()
+        _elevate_if_needed(ElevateType.DDCUTIL)
         _set_monitors(config, args.monitor, args.preset, args.check, args.verify)
 
 
